"""C H A T B O T    M E N U"""

from .addressbook import main as ab_main
from .notes import main as nb_main
from .sort import main as sr_main
from .information import start_info_menu
from .games_menu import main as games_main
from .logger import get_logger


logger = get_logger(__name__)


def main():
    """Main function"""

    while True:
        print(start_info_menu())
        user_input = input(f"\nEnter command {chr(10151)*3} ")
        if user_input == "1":
            ab_main()
        elif user_input == "2":
            nb_main()
        elif user_input == "3":
            sr_main()
        elif user_input == "4":
            games_main()
        elif user_input == "0":
            print(f"\n{chr(128075)} Good bay!")
            break
        else:
            print(f"\n{chr(129400)}I don't understand you!")


if __name__ == "__main__":
    logger.info('Start program')
    main()



"""M A G I C   L A Y E R"""

from random import choice
from rich.console import Console

console = Console()

answers = ['Безперечно', 'Вирішено', 'Жодних сумнівів', 'Хороші перспективи', 'Краще не розповідати',
           'За моїми даними - ні', 'Мені здається - так', 'Поки неясно, спробуй знову', 'Навіть не думай',
           'Найімовірніше', 'Запитай пізніше', 'Моя відповідь - ні', 'Безперечно так', 'Знаки кажуть - так',
           'Зараз не можна передбачити', 'Перспективи не дуже хороші',
           'Можеш бути впевнений у цьому', 'Так', 'Сконцентруйся і запитай знову', 'Дуже сумнівно']


def magic_layer():
    while True:
        input(f"\nЯ уважно слухаю {chr(10151) * 3} ")
        console.print(f"{chr(127744)} [green]{choice(answers)}[/green] {chr(127744)}")

        while True:
            print(f'\nу тебе є ще питання? (так/ні, yes/no)')
            user_input = input(f"{chr(10151) * 3} ")
            if user_input in ('ні', 'no'):
                print(f'Все буде Україна! {chr(9996)}\n')
                return False
            elif user_input in ('так', 'yes'):
                return magic_layer()
            else:
                continue


def main():
    console.print(f"\n{chr(129535)} [yellow]M A G I C   L A Y E R[/yellow] {chr(129535)}")
    print(f'\nЯк до тебе звертатися?')
    name = input(f"{chr(10151) * 3} ")
    print(f'Привіт {name.title()}! Я магічна куля, яке у тебе питання?.')

    magic_layer()


if __name__ == '__main__':
    main()



"""N O T E B O O K"""

from collections import UserDict
from .information import start_info_nb, help_info_nb
from .output import NoteBookOutput
from .prompt_tool_nb import Completer, RainbowLexer
from prompt_toolkit import prompt
import pickle
from .logger import get_logger


filename = "chat_data/notebook.bin"
logger = get_logger(__name__)


class NoteBook(UserDict):
    def add_record(self, record):
        self.data[record.name.value] = record

    def iterator(self, n=1):
        index = 1
        print_block = '-' * 100 + '\n'
        for record in self.data.values():
            print_block += str(record) + '\n'
            if index < n:
                index += 1
            else:
                yield print_block
                index, print_block = 1, '-' * 100 + '\n'
        yield print_block


class Filed:
    def __init__(self, value):
        self.value = value

    def __repr__(self):
        return f"{self.value}"


class Name(Filed):
    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, new_name):
        self._value = new_name.capitalize()


class Note(Filed):
    pass


class Tag(Filed):
    pass


class Record:
    def __init__(self, name, *notes):
        self.name = name
        self.notes = list(notes)
        self.tags = []

    def __repr__(self):
        return f'Title: {self.name}, Notes: {self.notes}, Tags: {self.tags}'

    def add_name(self, name):
        self.name = name

    def add_note(self, notes):
        self.notes.append(notes)

    def add_tag(self, tags):
        self.tags.append(tags)


def decor_error(func):
    """Decorator for handling exceptions """

    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except IndexError:
            return f"{chr(128679)} IndexError..."
        except KeyError:
            return f"{chr(128679)} KeyError..."
        except ValueError:
            return f"{chr(128679)} ValueError..."
        except AttributeError:
            return f"{chr(128679)} AttributeError..."

    return wrapper


@decor_error
def hello(*args, **kwargs: NoteBook):
    return f"{chr(129299)} How can I help you?\n"


@decor_error
def add_all_note(*args, **kwargs: NoteBook):
    """Added note in notebook"""

    nb = kwargs.get('nb')
    input_name = input(f"Enter name note {chr(10151) * 3} ")
    name = Name(input_name)
    rec = nb.get(name.value)
    if rec:
        return f"There is already a note with the name '{input_name}' "
    input_note = input(f"Enter note {chr(10151) * 3} ")
    input_tags = input(f"Enter tags {chr(10151) * 3} ")
    notes = Note(input_note)
    tags = Tag(input_tags)

    rec = Record(name, notes.value)
    rec.add_tag(tags.value)
    nb.add_record(rec)

    return f"add Name: '{name}', note: '{notes}', tag: '{tags}'"


@decor_error
def add_note(*args, **kwargs: NoteBook):
    """Added note in notebook"""

    nb = kwargs.get('nb')
    input_name = input(f"Enter name note {chr(10151) * 3} ")
    name = Name(input_name)
    rec = nb.get(name.value)
    if rec:
        input_note = input(f"Enter note {chr(10151) * 3} ")
        notes = Note(input_note)
        rec.add_note(notes.value)
    else:
        return f"There is no note with the name '{input_name}' "
    return f"add Name: '{name}', note: '{notes}'"


@decor_error
def del_note(*args, **kwargs: NoteBook):
    """Deletes note from notebook"""

    nb = kwargs.get('nb')
    input_name = input(f"Enter name note {chr(10151) * 3} ")
    name = Name(input_name)
    rec = nb.get(name.value)
    if rec:
        nb.pop(name.value)
        return f"{chr(9989)} Note '{name}' deleted {chr(10060)} "
    return f"{chr(10062)} Note '{name}' isn't in the NoteBook"


@decor_error
def add_tag(*args, **kwargs: NoteBook):
    """Added tag in note"""

    nb = kwargs.get('nb')
    input_name = input(f"Enter name note {chr(10151) * 3} ")
    name = Name(input_name)
    rec = nb.get(name.value)
    if rec:
        input_tags = input(f"Enter tags {chr(10151) * 3} ")
        tags = Tag(input_tags)
        rec.add_tag(tags.value)
    else:
        return f"{chr(10062)} Note '{name}' isn't in the NoteBook"
    return f'In note "{name}" added tag "{tags}"'


@decor_error
def find(*args, **kwargs: NoteBook):
    """Find notes that contain the specified text"""

    nb = kwargs.get('nb')
    sub = input(f"Enter request {chr(10151) * 3} ")
    for value in nb.values():
        value = str(value)
        if sub.lower() in value.lower():
            print(f'{"-" * 80}\n{value}')
        if not value:
            return f"{chr(10062)} On request '{sub}' not found in notebook"
    return f'{"-" * 80}\nOn request "{sub}", found the following notes'


@decor_error
def show_all(*args, **kwargs: NoteBook):
    """Display the contents of a NoteBook"""

    new = NoteBookOutput()
    return new.show_all(*args, **kwargs)


def exit_save_change(nb: NoteBook):
    """Request to save information"""

    while True:
        user_input_save = input(f"{chr(128221)}Save change? y/n: ")
        if user_input_save == "y":
            save_to_file(nb)
            break
        elif user_input_save == "n":
            break
        else:
            continue
    print(f"{chr(128075)} Good bye!")


def save_to_file(nb):
    """Save notes to the file"""

    with open(filename, "wb") as fh:
        pickle.dump(nb, fh)
        logger.info(f"Changes saved.")


def read_from_file():
    """Uploading data from a file"""

    try:
        with open(filename, "rb") as fh:
            return pickle.load(fh)
    except FileNotFoundError:
        return NoteBook()


"""Dictionary with commands(key - function: value - command)"""

COMMANDS = {
    hello: "hello",
    add_all_note: "add",
    add_note: "note",
    del_note: "del",
    add_tag: "tag",
    find: "find",
    show_all: "show",
    help_info_nb: "info"
}


def parser_command(user_input: str):
    """The function parses the string entered by the user, splits it into a command and other arguments"""

    for command, key_word in COMMANDS.items():
        if user_input.lower().split()[0] == key_word:
            return command, user_input.replace(key_word, "").strip().split()
    return None, None


def main():
    """Main function"""

    logger.info('Start Notebook')
    print(start_info_nb())
    nb = read_from_file()

    while True:
        user_input = prompt(f"\nEnter command {chr(10151) * 3} ", completer=Completer, lexer=RainbowLexer())
        if user_input:
            if user_input.lower() in ["close", "exit", ".", "0"]:
                exit_save_change(nb)
                break
            command, data = parser_command(user_input)
            if not command:
                print(f"\nSorry {chr(129400)}, I don't understand you!\n")
            else:
                print(command(*data, nb=nb))


if __name__ == "__main__":
    main()



"""L O G G E R"""

import logging

log_format = (
    "%(asctime)s [%(levelname)s] - %(name)s - %(funcName)15s:%(lineno)d - %(message)s"
)

file_handler = logging.FileHandler("chat_data/application.log")
file_handler.setLevel(logging.INFO)
file_handler.setFormatter(logging.Formatter(log_format))

stream_handler = logging.StreamHandler()
stream_handler.setLevel(logging.WARNING)
stream_handler.setFormatter(logging.Formatter(log_format))


def get_logger(name):
    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)
    logger.addHandler(stream_handler)
    logger.addHandler(file_handler)
    return logger



"""S E T U P"""

from setuptools import setup, find_namespace_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(name='Chatbot_by_IE',
      version='0.2.22',
      description='Chatbot: Addressbook, NoteBook, Sort file, Games',
      url='https://github.com/Sanyavas/chatbot-team-project.git',
      author='Oleksandr Vasylyna, Oleh Vakulchyk, Nataleia Orlovska, Anton Sokhnenko, Polina Dyka',
      author_email='vasilinaoleksanrd@gmail.com',
      license='MIT',
      classifiers=[
            "Programming Language :: Python :: 3",
            "License :: OSI Approved :: MIT License",
            "Operating System :: OS Independent"],
      long_description=long_description,
      # data_files=[('app.data', ['./app/data/application.log'])],
      include_package_data=True,
      packages=find_namespace_packages(),
      install_requires=["prompt_toolkit", "rarfile", "prettytable", "rich"],
      entry_points={"console_scripts": [
            "chatbot=app.menu:main"]}
      )



"""H A N G M A N"""

from random import choice
from rich.console import Console

console = Console()

word_list = ['слово', 'людина', 'час', 'справа', 'життя', 'день', 'рука', 'робота', 'місце', 'обличчя',
             'друг', 'питання', 'будинок', 'сторона', 'країна', 'світ', 'випадок', 'голова', 'дитина', 'сила',
             'кінець', 'система', 'частина', 'місто', 'ставлення', 'жінка', 'гроші', 'земля', 'машина', 'вода',
             'батько', 'проблема', 'година', 'право', 'нога', 'рішення', 'двері', 'образ', 'історія', 'влада', 'закон',
             'війна', 'бог', 'голос', 'тисяча', 'книга', 'можливість', 'результат', 'ніч', 'стіл', 'область',
             'стаття', 'число', 'компанія', 'народ', 'дружина', 'група', 'розвиток', 'процес', 'суд', 'умова',
             'засіб', 'початок', 'світло', 'час', 'шлях', 'душа', 'рівень', 'форма', "зв'язок", 'хвилина',
             'вулиця', 'вечір', 'якість', 'ідея', 'дорога', 'мати', 'дія', 'місяць', 'держава', 'мова', 'любов',
             'погляд', 'мама', 'століття', 'школа', 'ціль', 'суспільство', 'діяльність', 'організація', 'президент',
             'кімната', 'порядок', 'момент', 'театр', 'лист', 'ранок', 'допомога', 'ситуація', 'роль', 'рубль', 'сенс',
             'стан', 'квартира', 'орган', 'увага', 'тіло', 'праця', 'син', 'міра', 'смерть', 'ринок ', 'програма',
             'завдання', 'підприємство', 'вікно', 'розмова', 'уряд', 'виробництво', 'інформація', 'становище',
             'центр', 'відповідь', 'чоловік', 'автор', 'стіна', 'інтерес', 'федерація', 'правило', 'управління',
             'чоловік', 'ідея', 'партія', 'рада', 'рахунок', 'серце ', 'рух', 'матеріал', 'тиждень', 'почуття',
             'глава', 'наука', 'ряд', 'газета', 'причина', 'ціна', 'план', 'мова', 'точка', 'основа',
             'товариш', 'культура', 'думка', 'документ', 'інститут', 'хід', 'проект', 'зустріч', 'директор',
             'термін', 'палець', 'досвід', 'служба', 'доля', 'дівчина', 'черга', 'ліс', 'склад', 'кількість',
             "подія", "зал", "створення", "значення", "період", "крок", "брат", "мистецтво", "структура",
             "номер", "приклад", "дослідження", 'громадянин', 'гра', 'начальник', 'зріст', 'тема', 'принцип', 'метод',
             'фільм', 'краївка', 'гість', 'повітря', 'характер', 'боротьба', 'використання', 'розмір', 'освіта',
             'хлопчик', 'небо', 'армія', 'клас', 'представник', 'участь', 'дівчинка', 'політика',
             'герой', 'картина', 'долар', 'спина', 'територія', 'поле', 'зміна', 'напрямок', 'малюнок', 'течія',
             'церква', 'банк', 'сцена', 'населення', 'більшість', 'музика', 'правда', 'свобода', 'команда',
             'союз', 'лікар', 'договір', 'дерево', 'факт', 'господар', 'природа', 'телефон', 'позиція', 'двір',
             'письменник', 'літак', 'обсяг', 'рід', 'сонце', 'віра', 'берег', 'вистава', 'фірма', 'завод',
             'колір', 'журнал', 'керівник', 'фахівець', 'оцінка', 'регіон', 'пісня', 'відсоток', 'батько', 'море',
             'вимоги', 'основа', 'половина', 'роман', 'коло', 'аналіз ', 'вірші', 'автомобіль', 'економіка',
             'література', 'папір', 'поет', 'ступень', 'пан', 'надія', 'предмет', 'варіант', 'міністр', 'кордон',
             'дух', 'модель', 'операція', 'пара', 'сон', 'назва', 'розум', 'привід', 'старий', 'мільйон', 'успіх',
             'щастя', 'хлопці', 'кабінет', 'магазин', 'простір', 'вихід', 'удар', 'база', 'знання', 'текст', 'захист',
             'керівництво', 'площа', 'свідомість', 'вік', 'учасник', 'дільниця', 'пункт', 'лінія', 'бажання', 'тато',
             'лікар', 'губа', 'дочка', 'середовище', 'голова', 'подання', 'солдат', 'художник', 'зброя',
             'відповідність', 'вітер', 'хлопець', 'зір', 'генерал', 'вогонь', 'поняття', 'будівництво', 'вухо',
             'груди', 'ніс', 'страх', 'послуга', 'зміст', 'радість', 'безпека', 'продукт', 'комплекс', 'бізнес', 'сад',
             'співробітник', 'літо', 'курс', 'пропозиція', 'рот', 'технологія', 'реформа', 'відсутність', 'собака',
             'камінь', 'майбутнє', 'оповідання', 'контроль', 'річка', 'продукція', 'сума', 'техніка', 'будівля',
             'сфера', 'необхідність', 'фонд', 'підготовка', 'лист', 'республіка', 'господарство', 'воля', 'бюджет',
             'сніг', 'село', 'елемент', 'обставина', 'німець', 'перемога', 'джерело', "зірка", "вибір",
             "маса", 'підсумок', 'сестра', 'практика', 'проведення', 'кишеня', 'слава', 'кухня', 'визначення',
             'функція', 'військо', 'комісія', 'застосування', 'капітан ', 'працівник', 'забезпечення', 'офіцер',
             'прізвище', 'межа', 'вибори', 'вчений', 'пляшка', 'бій', 'теорія', 'зона', 'відділ', 'зуб', 'розробка',
             'особа', 'гора', 'товар', 'метр', 'свято', 'вплив', 'читач', 'задоволення', 'актор', 'сльоза',
             'відповідальність ', 'вчитель', 'акт', 'біль', 'множина', 'особливість', 'показник', 'корабель', 'звук',
             'враження', 'приватність', 'дитинство', 'висновок', 'професор', 'частка', 'норма', 'минуле', 'командир',
             'коридор', 'підтримка', 'рамка', 'ворог', 'етап', 'чорт', 'дід', 'збори ', 'хвороба', 'клітина',
             'шкіра', 'заява', 'спроба', 'порівняння', 'розрахунок', 'депутат', 'комітет', 'знак', 'дядько', 'облік',
             'хліб', 'чай', 'режим', 'ціле', 'вірус', 'вираз', 'зима', 'десяток', 'глибина', 'мережа',
             'студент ', 'секунда', 'швидкість', 'пошук', 'сутність', 'податок', 'помилка', 'дохід']  # список слів


# Генератор рандомного слова

def get_word():
    return choice(word_list).upper()


# функція перевірка літер та заміни на "_"

def print_close_word(word_, list_):
    for c in word_:
        if c in list_:
            console.print(f"[green]{c}[/green]", end=' ')
        else:
            print('_', end=' ')
    print()


# Функція нова гра чи вихід

def game_again():
    while True:
        print(f'\nХочеш зіграти ще? так або 1/ні або 0')
        answer = input(f"{chr(10151) * 3} ")
        if answer == 'ні' or answer == "0":
            console.print("Бувай!\n[yellow]Слава Україні![/yellow]\n")
            break
        elif answer == 'так' or answer == "1":
            return main()
        else:
            continue


def main():
    word = get_word()
    word_completion = '_ ' * len(word)  # строка, у вигляді "_" замість літер
    list_letters = []  # список названих літер
    list_words = []  # список названих слів
    count_letter = 0  # кількість правильних літер
    attempt = 6  # кількість спроб
    console.print(f"\n[yellow]H A N G M A N[/yellow]")
    console.print(f'\n[green]Ви маєте {attempt} спроб вгадати слово {display_hangman(attempt)}[/green]')
    print(f'Слово з {len(word)} літер: {word_completion}')

    while True or attempt < 1:
        print(f'Введіть літеру або слово')
        user_in = input(f"{chr(10151) * 3} ").upper()
        if not user_in.isalpha():  # перевірка чи не цифра
            console.print(f"[red]Мають бути лише літери![/red]")
            continue

        if user_in == "інфо".upper():  # інформація які літери та слова називали
            console.print(f'названі літери: [green]{", ".join(list_letters)}[/green] '
                          f'\nназвані слова: [green]{", ".join(list_words)}[/green]')
            continue

        if user_in in list_words or user_in in list_letters:  # перевірка на вже названі літери та слова
            console.print(f"Ви вже [red]{user_in}[/red] називали!")
            continue

        if user_in not in word:
            attempt -= 1
            if len(user_in) == 1:
                list_letters.append(user_in)
            if len(user_in) > 1:
                list_words.append(user_in)
            console.print(f'\n[yellow]Ви маєте {attempt} спроб вгадати слово {display_hangman(attempt)}[/yellow]')
            print_close_word(word, list_letters)

        for letter in word:
            if user_in == letter:
                count_letter += 1
                if count_letter != len(word):
                    list_letters.append(user_in)
                    print_close_word(word, list_letters)
                    print(f'Ви маєте {attempt} спроб вгадати слово\n')

        if user_in == word or count_letter == len(word):  # Перемога
            console.print(f'\n[yellow]Вітаю! {chr(127881)} Ви вгадали слово:[/yellow]  [green]{word}[/green]')
            game_again()
            break

        if attempt == 0:  # Поразка
            console.print(f'[red]Ви програли :(  {display_hangman(attempt)} слово було:[/red] [blue]{word}[/blue]')
            game_again()
            break


# функція поточного стану шибиниці

def display_hangman(attempt):
    stages = [  # фінальний стан: голова, тулуб, обидві руки, обидві ноги
        '''
           --------
           |      |
           |      O
           |     \\|/
           |      |
           |     / \\
           -
        ''',
        # голова, тулуб, обидві руки, одна нога
        '''
           --------
           |      |
           |      O
           |     \\|/
           |      |
           |     / 
           -
        ''',
        # голова, тулуб, обидві руки
        '''
           --------
           |      |
           |      O
           |     \\|/
           |      |
           |      
           -
        ''',
        # голова, тулуб и одна рука
        '''
           --------
           |      |
           |      O
           |     \\|
           |      |
           |     
           -
        ''',
        # голова и тулуб
        '''
           --------
           |      |
           |      O
           |      |
           |      |
           |     
           -
        ''',
        # голова
        '''
           --------
           |      |
           |      O
           |    
           |      
           |     
           -
        ''',
        # початковий стан
        '''
           --------
           |      |
           |      
           |    
           |      
           |     
           -
        '''
    ]
    return stages[attempt]


if __name__ == "__main__":
    main()



"""G A M E S   M E N U"""

from .information import start_info_games
from .games.magic_layer import main as mg_main
from .games.hangman import main as hm_main
from .logger import get_logger


logger = get_logger(__name__)


def main():

    logger.info('Start Games')
    while True:
        print(start_info_games())
        user_input = input(f"\nEnter command {chr(10151) * 3} ")
        if user_input == "1":
            mg_main()
        elif user_input == "2":
            hm_main()
        elif user_input == "0":
            print(f"\n{chr(128075)} Good bay!")
            break
        else:
            print(f"\n{chr(129400)}I don't understand you!")


if __name__ == '__main__':
    main()
